<?php
declare(strict_types=1);

namespace ForteEcom\Payments\Controller\Response;

use ForteEcom\Payments\Model\Ui\ConfigProvider;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Data\Form\FormKey\Validator;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Framework\View\Result\Page;
use Magento\Framework\View\Result\PageFactory;
use Magento\Vault\Api\Data\PaymentTokenFactoryInterface;
use Magento\Vault\Api\Data\PaymentTokenInterface;
use Magento\Vault\Api\PaymentTokenManagementInterface;
use Magento\Vault\Api\PaymentTokenRepositoryInterface;

/**
 * implementing post interfaces so that it can handle requests from outside
 *
 * @link https://devdocs.magento.com/guides/v2.3/extension-dev-guide/routing.html
 *
 * Class Index
 */
class AddCard extends Action implements HttpPostActionInterface
{
    /**
     * @var PageFactory
     */
    protected $pageFactory;
    protected $token_factory;
    protected $token_repository;
    protected $encryptor;
    protected $config_provider;
    protected $json_factory;
    protected $token_management;
    protected $form_key_validator;

    /**
     * @param Context                         $context
     * @param ConfigProvider                  $configProvider
     * @param Validator                       $form_key_validator
     * @param PageFactory                     $pageFactory
     * @param PaymentTokenFactoryInterface    $tokenFactory
     * @param PaymentTokenRepositoryInterface $tokenRepository
     * @param EncryptorInterface              $encryptor
     * @param JsonFactory                     $jsonFactory
     * @param PaymentTokenManagementInterface $tokenManagement
     */
    public function __construct(
        Context $context,
        ConfigProvider $configProvider,
        Validator $form_key_validator,
        PageFactory $pageFactory,
        PaymentTokenFactoryInterface $tokenFactory,
        PaymentTokenRepositoryInterface $tokenRepository,
        EncryptorInterface $encryptor,
        JsonFactory $jsonFactory,
        PaymentTokenManagementInterface $tokenManagement
    )
    {
        parent::__construct($context);

        $this->form_key_validator = $form_key_validator;
        $this->config_provider    = $configProvider;
        $this->token_factory      = $tokenFactory;
        $this->token_repository   = $tokenRepository;
        $this->pageFactory        = $pageFactory;
        $this->encryptor          = $encryptor;
        $this->json_factory       = $jsonFactory;
        $this->token_management   = $tokenManagement;
    }

    /**
     * @return ResponseInterface|ResultInterface|Page
     */
    public function execute()
    {

        if ($this->form_key_validator->validate($this->getRequest())) {
            $data        = $this->getRequest()->getParams()['data'];
            $customer_id = $data['customer_id'];

            $payment_token = $this->token_factory->create(PaymentTokenFactoryInterface::TOKEN_TYPE_CREDIT_CARD);
            $payment_token->setExpiresAt(strtotime($data['card_expiry_year'] . '-' . $data['card_expiry_month'] . '-01'));
            $payment_token->setGatewayToken($data['card_token']);

            $payment_token->setIsActive(true);
            $payment_token->setIsVisible(true);
            $payment_token->setPaymentMethodCode($this->config_provider::CC_VAULT_CODE);
            $payment_token->setCustomerId($customer_id);
            $public_hash = $this->generatePublicHash($payment_token);
            $payment_token->setTokenDetails(json_encode([
                'type'            => $data['card_type'],
                'last4'           => $data['card_last4'],
                'expiration_date' => $data['card_expiry_month'] . '/' . $data['card_expiry_year'],
                'public_hash'     => $public_hash
            ]));

            $existing_token = $this->token_management->getByPublicHash($public_hash, $customer_id);
            if ($existing_token == null) {
                $payment_token->setPublicHash($public_hash);
                $this->token_repository->save($payment_token);
                $message = __('Credit card details saved');
                $this->messageManager->addSuccessMessage($message);
            } else {
                if ($existing_token->getIsVisible()) {
                    $message = __('Already saved card');
                    $this->messageManager->addErrorMessage($message);
                } else {
                    $message = __('Credit card details saved');
                    $this->messageManager->addSuccessMessage($message);
                }
                $existing_token->setIsVisible(true);
                $existing_token->setIsActive(true);
                $this->token_repository->save($existing_token);
            }

            $response           = [
                'success' => true,
                'message' => $message
            ];
            $http_response_code = 200;

        } else {
            $message            = __('Could not validate request');
            $response           = [
                'success' => false,
                'message' => $message
            ];
            $http_response_code = 422;
            $this->messageManager->addErrorMessage($message);
        }

        return $this->json_factory->create()->setData($response)->setHttpResponseCode($http_response_code);
    }

    /**
     * Generate vault payment public hash
     *
     * @param PaymentTokenInterface $paymentToken
     *
     * @return string
     */
    protected function generatePublicHash(PaymentTokenInterface $paymentToken)
    {
        $hashKey = $paymentToken->getGatewayToken();
        if ($paymentToken->getCustomerId()) {
            $hashKey = $paymentToken->getCustomerId();
        }

        $hashKey .= $paymentToken->getPaymentMethodCode()
            . $paymentToken->getType()
            . $paymentToken->getGatewayToken()
            . $paymentToken->getExpiresAt();

        return $this->encryptor->getHash($hashKey);
    }

}
