<?php
declare(strict_types=1);

namespace ForteEcom\Payments\Controller\Response;

use Exception;
use ForteEcom\Payments\Model\Payment\ForteEcomHelper;
use ForteEcom\Payments\Model\Ui\ConfigProvider;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\Result\Json;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Data\Form\FormKey\Validator;
use Magento\Framework\View\Result\PageFactory;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface;

/**
 * implementing post interfaces so that it can handle requests from outside
 *
 * @link https://devdocs.magento.com/guides/v2.3/extension-dev-guide/routing.html
 *
 * Class Index
 */
class InitPayment extends Action implements HttpPostActionInterface
{
    /**
     * @var PageFactory
     */
    protected $page_factory;
    protected $json_factory;
    protected $transaction_builder;
    protected $config_provider;
    protected $form_key_validator;

    /**
     * @param Context          $context
     * @param ConfigProvider   $config_provider
     * @param Validator        $form_key_validator
     * @param PageFactory      $pageFactory
     * @param JsonFactory      $jsonFactory
     * @param BuilderInterface $transactionBuilder
     */
    public function __construct(
        Context $context,
        ConfigProvider $config_provider,
        Validator $form_key_validator,
        PageFactory $pageFactory,
        JsonFactory $jsonFactory,
        BuilderInterface $transactionBuilder
    )
    {
        parent::__construct($context);

        $this->form_key_validator  = $form_key_validator;
        $this->config_provider     = $config_provider;
        $this->page_factory        = $pageFactory;
        $this->json_factory        = $jsonFactory;
        $this->transaction_builder = $transactionBuilder;
    }

    /**
     * @return ResponseInterface|Json|ResultInterface
     * @throws Exception
     */
    public function execute()
    {
        if ($this->form_key_validator->validate($this->getRequest())) {

            $request  = $this->getRequest()->getParams();
            $order_id = $request['order_id'];

            /** @var Order $order */
            $order             = ObjectManager::getInstance()->create('\Magento\Sales\Model\Order')->load($order_id);
            $payment           = $order->getPayment();
            $order_description = sprintf('%s - Order: %s', $order->getStoreName(), $order->getIncrementId());
            $amount            = $order->getGrandTotal();
            $config            = $this->config_provider->getConfig();
            $current_data      = $payment->getAdditionalInformation();

            $status_result = ForteEcomHelper::get_payment_status($current_data['_forteecom_merchant_id'], $current_data['_forteecom_order_id'], $current_data['_forteecom_session_id']);
            ForteEcomHelper::update_payment_status($order, $payment, $status_result['_forteecom_order_status'], $current_data['_forteecom_order_status'], $this->transaction_builder);

            if (in_array($status_result['_forteecom_order_status'], [ForteEcomHelper::STATUS_DECLINE, ForteEcomHelper::STATUS_EXPIRE, ForteEcomHelper::STATUS_CANCEL, ForteEcomHelper::STATUS_ERROR])) {
                $init_result = ForteEcomHelper::init_payment($config['merchant_id'], $config['pre_auth'], $amount, $order_id, $order_description, 'KZT');

                $payment->setAdditionalInformation('_forteecom_order_id', $init_result['_forteecom_order_id']);
                $payment->setAdditionalInformation('_forteecom_order_status', ForteEcomHelper::STATUS_INIT);
                $payment->setAdditionalInformation('_forteecom_session_id', $init_result['_forteecom_session_id']);
                $payment->setAdditionalInformation('_forteecom_merchant_id', $config['merchant_id']);

                $order->save();
                $payment->save();

                $response           = [
                    'orderId'   => $init_result['_forteecom_order_id'],
                    'sessionId' => $init_result['_forteecom_session_id']
                ];
                $http_response_code = 200;

            } else {
                $response           = [
                    'sessionId' => $current_data['_forteecom_session_id'],
                    'orderId'   => $current_data['_forteecom_order_id'] 
                ];
                $http_response_code = 200;
            }

        } else {
            $message            = __('Could not validate request');
            $response           = [
                'sessionId' => $current_data['_forteecom_session_id'],
                'orderId'   => $current_data['_forteecom_order_id'] 
            ];
            $http_response_code = 422;
            $this->messageManager->addErrorMessage($message);
        }

        return $this->json_factory->create()->setData($response)->setHttpResponseCode($http_response_code);
    }

}
