<?php

namespace ForteEcom\Payments\Model\Payment;

use Exception;
use ForteEcom\Payments\Model\Ui\ConfigProvider;
use Magento\Checkout\Model\Session;
use Magento\Directory\Helper\Data as DirectoryHelper;
use Magento\Framework\Api\AttributeValueFactory;
use Magento\Framework\Api\ExtensionAttributesFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\PaymentException;
use Magento\Framework\Message\ManagerInterface;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Registry;
use Magento\Payment\Helper\Data;
use Magento\Payment\Model\InfoInterface;
use Magento\Payment\Model\Method\AbstractMethod;
use Magento\Payment\Model\Method\Logger;
use Magento\Sales\Api\Data\OrderPaymentInterface;
use Magento\Sales\Model\Order\Payment\Transaction;
use Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface;
use Magento\Vault\Api\Data\PaymentTokenFactoryInterface;
use Magento\Vault\Api\PaymentTokenManagementInterface;
use Magento\Vault\Model\VaultPaymentInterface;

class ForteEcom extends AbstractMethod implements VaultPaymentInterface
{
    protected $_code = "forteecom";
    protected $_isOffline = false;
    protected $_isGateway = true;
    protected $_canAuthorize = true;
    protected $_canCapture = true;
    protected $_isInitializeNeeded = false;
    protected $_supportedCurrencyCodes = ["KZT"];
    protected $_canRefund = true;
    protected $_canRefundInvoicePartial = true;
    protected $transaction_builder;
    protected $token_factory;
    protected $checkout_session;
    protected $config_provider;
    protected $token_management;
    protected $message_manager;

    public function __construct(
        PaymentTokenFactoryInterface $tokenFactory,
        Session $checkoutSession,
        ManagerInterface $messageManager,
        ConfigProvider $configProvider,
        Context $context,
        Registry $registry,
        ExtensionAttributesFactory $extensionFactory,
        AttributeValueFactory $customAttributeFactory,
        PaymentTokenManagementInterface $tokenManagement,
        Data $paymentData,
        ScopeConfigInterface $scopeConfig,
        BuilderInterface $transaction_builder,
        Logger $logger,
        AbstractResource $resource = null,
        AbstractDb $resourceCollection = null,
        array $data = [],
        DirectoryHelper $directory = null
    )
    {
        $this->message_manager     = $messageManager;
        $this->token_factory       = $tokenFactory;
        $this->transaction_builder = $transaction_builder;
        $this->checkout_session    = $checkoutSession;
        $this->config_provider     = $configProvider;
        $this->token_management    = $tokenManagement;
        parent::__construct($context, $registry, $extensionFactory, $customAttributeFactory, $paymentData, $scopeConfig, $logger, $resource, $resourceCollection, $data, $directory);
    }

    /**
     * Capture payment that was invoiced and had forteecom status on-hold
     *
     * @inheritDoc
     */
    public function capture(InfoInterface $payment, $amount)
    {
        $current_data = $payment->getAdditionalInformation();
        $this->config_provider->getConfig();
        if ($current_data['_forteecom_order_status'] != ForteEcomHelper::STATUS_ON_HOLD) {
            throw new PaymentException(__('ForteEcom: could not capture payment, because payment is not "Authorized" state'));
        }

        $complete_result = ForteEcomHelper::complete_payment($current_data['_forteecom_merchant_id'], $current_data['_forteecom_order_id'], $current_data['_forteecom_session_id']);

        if (!empty($complete_result['error_message'])) {
            throw new LocalizedException(__('ForteEcom: could not capture payment. Error: ' . $complete_result['error_message']));
        }

        /** @var Transaction $authorization_transaction */
        if ($authorization_transaction = $payment->getAuthorizationTransaction()) {
            $authorization_transaction->setIsClosed(1);
            $authorization_transaction->save();
            $payment->setAdditionalInformation('_forteecom_order_status', ForteEcomHelper::STATUS_SUCCESS);
            $payment->save();
        }

        return $this;
    }


    /**
     * Handle payment placed in checkout using saved payment method in vault
     *
     * @inheritDoc
     * @throws Exception
     */
    public function authorize(InfoInterface $payment, $amount)
    {
        $this->config_provider->getConfig();
        /** @var OrderPaymentInterface $payment */

        $additional_information = $payment->getAdditionalInformation();
        $payment->unsAdditionalInformation('public_hash');

        // Unset additional info in quote for security reasons
        $quote_payment = $this->checkout_session->getQuote()->getPayment();
        $quote_payment->unsAdditionalInformation('public_hash');
        $quote_payment->save();

        if (!empty($additional_information['public_hash'])) {

            $object_manager = ObjectManager::getInstance();
            /** @var \Magento\Customer\Model\Session $customer_session */
            $customer_session = $object_manager->get('Magento\Customer\Model\Session');
            $customer         = $customer_session->getCustomer();

            $order_id          = '';
            $order_description = sprintf('Customer: %s', $customer->getName());
            $amount            = $payment->getAmountOrdered();
            $config            = $this->config_provider->getConfig();
            $payment_token     = $this->token_management->getByPublicHash($additional_information['public_hash'], $customer->getId());

            $init_result = ForteEcomHelper::init_payment($config['merchant_id'], $config['pre_auth'], $amount, $order_id, $order_description, 'KZT');
            if (!empty($init_result['error_message'])) {
                throw new PaymentException(__('Initialization process failed on Gateway'));
            }

            $payment->setAdditionalInformation('_forteecom_order_id', $init_result['_forteecom_order_id']);
            $payment->setAdditionalInformation('_forteecom_order_status', ForteEcomHelper::STATUS_INIT);
            $payment->setAdditionalInformation('_forteecom_session_id', $init_result['_forteecom_session_id']);
            $payment->setAdditionalInformation('_forteecom_merchant_id', $config['merchant_id']);
            $purchase_result = ForteEcomHelper::purchase_by_token($config['merchant_id'], $init_result['_forteecom_order_id'], $init_result['_forteecom_session_id'], $amount, $order_id, $order_description, 398, $payment_token);

            if (!empty($purchase_result['error_message'])) {
                throw new PaymentException(__($purchase_result['error_message']));
            }

            $payment->setAdditionalInformation('_forteecom_order_status', $purchase_result['_forteecom_order_status']);
            $payment->setAdditionalInformation('is_vault_used', true);
            $payment->setAdditionalInformation('payment_token_id', $payment_token->getEntityId());

        } else {
            $payment->setAdditionalInformation('is_vault_used', false);
        }
    }

    /**
     * Refund specified amount for payment
     *
     * @param InfoInterface $payment
     * @param float $amount
     * @return $this
     *
     */
    public function refund(InfoInterface $payment, $amount) 
    {
        $this->config_provider->getConfig();
        $current_data = $payment->getAdditionalInformation();
        if (!in_array($current_data['_forteecom_order_status'], [ForteEcomHelper::STATUS_SUCCESS, ForteEcomHelper::STATUS_REFUND])) {
            throw new PaymentException(__('ForteEcom: could not refund payment, because payment is in wrong state'));
        }

        $refund_result = ForteEcomHelper::refund_payment($current_data['_forteecom_merchant_id'], $current_data['_forteecom_order_id'], $current_data['_forteecom_session_id'], $amount, 'KZT');

        if (!empty($refund_result['error_message'])) {
            throw new LocalizedException(__('ForteEcom: could not refund payment. Error: ' . $refund_result['error_message']));
        }
        $order = $payment->getOrder();
        ForteEcomHelper::create_transaction($order, $this->transaction_builder, $current_data['_forteecom_order_id'], Transaction::TYPE_REFUND, 1);
    }

    /**
     * @inheritDoc
     */
    public function canUseForCurrency($currencyCode)
    {
        if (!in_array($currencyCode, $this->_supportedCurrencyCodes)) return false;
        return parent::canUseForCurrency($currencyCode);
    }

    /**
     * @inheritDoc
     */
    public function getProviderCode()
    {
        return $this->_code;
    }

}
