<?php

namespace ForteEcom\Payments\Model\Payment;


use Exception;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Message\ManagerInterface;
use Magento\Sales\Api\Data\OrderPaymentInterface;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Payment\Transaction;
use Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface;
use Magento\Vault\Api\Data\PaymentTokenInterface;

class ForteEcomHelper
{
    //const HOST = "https://ecom-merchant-api.fortebank.com";
    //const HOST_DEMO = "https://qa-ecom-merchant-api.fortebank.com";
	const HOST = "https://qa-merchant-go.fortebank.com";
    const HOST_DEMO = "https://qa-merchant-go.fortebank.com";

    const ENDPOINT_INIT = "/init";
    const ENDPOINT_STATUS = "/status";
    const ENDPOINT_COMPLETE = "/complete";
    const ENDPOINT_PURCHASE = "/purchase";
    const ENDPOINT_PURCHASE_TOKEN = "/token/purchase";
    const ENDPOINT_REFUND = "/refund";
    const ENDPOINT_REVERSE = "/reverse";

    const ENDPOINT_RESPONSE_CODE = "https://adminapi.forte.kz/codes";

    const STATUS_INIT = 'CREATED';
    const STATUS_LOCK = 'ON_LOCK';
    const STATUS_PROCESS = 'ON_PAYMENT';
    const STATUS_SUCCESS = 'APPROVED';
    const STATUS_DECLINE = 'DECLINED';
    const STATUS_ON_HOLD = 'PREAUTH_APPROVED';
    const STATUS_REVERSE = 'REVERSED';
    const STATUS_REFUND = 'REFUNDED';
    const STATUS_EXPIRE = 'EXPIRED';
    const STATUS_CANCEL = 'CANCELED';
    const STATUS_ERROR = 'ERROR';

    const AVAILABLE_CURRENCIES = [
        'KZT' => [
            'id'   => 398,
            'code' => 'KZT',
            'name' => 'Kazakhstani tenge'
        ],
    ];

    public static function init_payment($_forteecom_merchant_id, $pre_auth, $amount, $remote_id, $description, $currency_code)
    {
        $currency_id = self::get_currency_id($currency_code);
        $data        = [
            'merchantId' => $_forteecom_merchant_id,
            'preAuth'    => $pre_auth,
            'source'     => [
                'jsonClass'   => 'TransactionSourcePurchase',
                'amount'      => (float)$amount,
                'currency'    => (string)$currency_id,
                'remoteId'    => $remote_id,
                'description' => $description,
            ]
        ];

		
        $request_body = json_encode($data);

        $request_result = self::send_request(self::ENDPOINT_INIT, $request_body);
		

        $_forteecom_order_id   = null;
        $_forteecom_session_id = null;
        if (empty($request_result['error'])) {
            if ($request_result['response']->responseCode == 0) {
                $_forteecom_order_id   = $request_result['response']->data->orderId;
                $_forteecom_session_id = $request_result['response']->data->sessionId;
            } else {
                $request_result['error'] = $request_result['response']->responseMessage;
            }
        }


        return [
            'error_message'         => $request_result['error'],
            '_forteecom_order_id'   => $_forteecom_order_id,
            '_forteecom_session_id' => $_forteecom_session_id,
        ];
    }

    public static function get_currency_id($currency_code)
    {
        return self::AVAILABLE_CURRENCIES[$currency_code]['id'];
    }

    public static function send_request($endpoint, $body)
    {
        $curl = curl_init();
        $url = self::get_url($endpoint);
        curl_setopt_array($curl, [
            CURLOPT_URL            => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_SSL_VERIFYHOST => 0,
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_ENCODING       => '',
            CURLOPT_MAXREDIRS      => 10,
            CURLOPT_TIMEOUT        => 30,
            CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST  => 'POST',
            CURLOPT_POSTFIELDS     => $body,
            CURLOPT_HTTPHEADER     => [
                'Content-Type: application/json',
            ]
        ]);

        $response = json_decode(curl_exec($curl));
        $error    = curl_error($curl);
        curl_close($curl);

        if (empty($error) && !property_exists($response, 'responseCode')) {
            $error = property_exists($response, 'moreInfo') ? $response->moreInfo : null;
        }

        return [
            'response' => $response,
            'error'    => $error
        ];
    }

    /**
     * @param string                $_forteecom_merchant_id
     * @param string                $_forteecom_order_id
     * @param string                $_forteecom_session_id
     * @param float                 $amount
     * @param int                   $remote_id
     * @param string                $description
     * @param int                   $currency_id
     * @param PaymentTokenInterface $payment_token
     *
     * @return array
     */
    public static function purchase_by_token($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id, $amount, $remote_id, $description, $currency_id, $payment_token)
    {
        $expires_at = strtotime($payment_token->getExpiresAt());
        $data       = array(
            'merchantId' => $_forteecom_merchant_id,
            'orderId'    => $_forteecom_order_id,
            'sessionId'  => $_forteecom_session_id,
            'ip'         => $_SERVER['REMOTE_ADDR'],
            'token'      =>$payment_token->getGatewayToken(),
            'source'     => array(
                'jsonClass'   => "TransactionSourcePurchase",
                'amount'      => (float)$amount,
                'currency'    => (string)$currency_id,
                'remoteId'    => $remote_id,
                'description' => $description
            ),
        );

        $request_body   = json_encode($data);
        $request_result = self::send_request(self::ENDPOINT_PURCHASE_TOKEN, $request_body);

        $_forteecom_order_status = null;
        $_forteecom_order_data   = null;
        if (empty($request_result['error'])) {
            $_forteecom_order_status = $request_result['response']->responseMessage;
            if ($request_result['response']->responseCode != 0) {
                $_forteecom_order_data   = $request_result['response']->data->purchaseResponseCode;
                $request_result['error'] = self::get_response_message($request_result['response']->data->purchaseResponseCode);
            } else {
                $_forteecom_order_data = $request_result['response']->data->result;
            }
        }

        return array(
            'error_message'           => $request_result['error'],
            '_forteecom_order_status' => $_forteecom_order_status,
            '_forteecom_order_data'   => $_forteecom_order_data,
        );

    }

    /**
     * Get localized message for error
     *
     * @param string $code
     *
     * @return string
     */
    public static function get_response_message($code)
    {
        $store_locale = ObjectManager::getInstance()->get('Magento\Framework\Locale\Resolver')->getLocale();
        switch ($store_locale) {
            case 'ru_RU':
                $lang = '_ru';
                break;
            case 'kk_KZ':
                $lang = '_kk';
                break;
            default:
                $lang = '';
                break;
        }

        $url = self::ENDPOINT_RESPONSE_CODE . '?' . http_build_query(['name' => $code]);

        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL            => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_SSL_VERIFYHOST => 0,
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_ENCODING       => '',
            CURLOPT_MAXREDIRS      => 10,
            CURLOPT_TIMEOUT        => 30,
            CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1
        ));

        $response = json_decode(curl_exec($curl), true);
        $error    = curl_error($curl);
        curl_close($curl);

        return $response[0]['message' . $lang];
    }

    public static function get_payment_status($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id)
    {
        $data = [
            'merchantId' => $_forteecom_merchant_id,
            'orderId'    => $_forteecom_order_id,
            'sessionId'  => $_forteecom_session_id,
        ];

        $request_body = json_encode($data);

        $request_result = self::send_request(self::ENDPOINT_STATUS, $request_body);

        $_forteecom_order_status = null;
        $_forteecom_add_info     = null;
        if (empty($request_result['error'])) {
            $_forteecom_order_status = $request_result['response']->data->orderStatus;
            $_forteecom_add_info     = $request_result['response']->data->addInfo;
        }

        return [
            'error_message'           => $request_result['error'],
            '_forteecom_order_status' => $_forteecom_order_status,
            '_forteecom_add_info'     => $_forteecom_add_info,
        ];
    }

    public static function complete_payment($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id)
    {
        $data = [
            'merchantId' => $_forteecom_merchant_id,
            'orderId'    => $_forteecom_order_id,
            'sessionId'  => $_forteecom_session_id,
        ];

        $request_body = json_encode($data);

        $request_result = self::send_request(self::ENDPOINT_COMPLETE, $request_body);

        $_forteecom_approval_code = null;
        if (empty($request_result['error'])) {
            $_forteecom_approval_code = $request_result['response']->data->result->approvalCode;
        }

        return [
            'error_message'            => $request_result['error'],
            '_forteecom_approval_code' => $_forteecom_approval_code,
        ];
    }

    public static function refund_payment($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id, $amount, $currency_code)
    {
        $currency_id = self::get_currency_id($currency_code);
        $data        = array(
            'orderIdent' => array(
                'merchantId' => $_forteecom_merchant_id,
                'orderId'    => $_forteecom_order_id,
                'sessionId'  => $_forteecom_session_id,
            ),
            'transaction' => array(
                'jsonClass'   => "TransactionSourceRefund",
                'amount'      => (float)$amount,
                'currency'    => (string)$currency_id
            ),
        );

        $request_body = json_encode($data);

        $request_result = self::send_request(self::ENDPOINT_REFUND, $request_body);

        $_forteecom_message = null;
        if (empty($request_result['error'])) {
            $_forteecom_message = (isset($request_result['response']->message)) ? $request_result['response']->message : 'Success';
        } else {
            $request_result['error'] = 'Error occured on refund';
        }

        return [
            'error_message'      => $request_result['error'],
            '_forteecom_message' => $_forteecom_message,
        ];
    }

    /**
     * @param Order                 $order
     * @param OrderPaymentInterface $payment
     * @param string                $new_status
     * @param string                $old_status
     * @param BuilderInterface      $transaction_builder
     * @param ManagerInterface|null $message_manager
     *
     * @throws Exception
     */
    public static function update_payment_status(&$order, &$payment, $new_status, $old_status, $transaction_builder)
    {
        if ($old_status == $new_status) {
            return;
        }
        $_forteecom_order_id = $payment->getAdditionalInformation()['_forteecom_order_id'];
        $payment->setAdditionalInformation('_forteecom_order_status', $new_status);
        $order->setCustomerNote('Payment status ' . $new_status);
        $exception       = null;
        $message_error   = null;
        $message_success = null;

        switch ($new_status) {
            case self::STATUS_SUCCESS:
                $payment->setBaseAmountPaid($payment->getBaseAmountOrdered());
                $payment->setAmountPaid($payment->getAmountOrdered());
                $order->setTotalPaid($payment->getAmountOrdered());
                $order->setBaseTotalPaid($payment->getBaseAmountOrdered());
                $transaction = self::create_transaction($order, $transaction_builder, $_forteecom_order_id, Transaction::TYPE_CAPTURE, 1);
                $order->prepareInvoice()
                    ->setState(Order\Invoice::STATE_PAID)
                    ->setRequestedCaptureCase(Order\Invoice::CAPTURE_ONLINE)
                    ->setTransactionId($transaction->getId())
                    ->register()
                    ->save();
                $order->setTotalInvoiced($order->getGrandTotal());
                $new_state = Order::STATE_PROCESSING;
                break;
            case self::STATUS_DECLINE:
                $order->cancel();
                $new_state = Order::STATE_CANCELED;
                break;
            case self::STATUS_EXPIRE:
                $new_state = Order::STATE_CLOSED;
                break;
            case self::STATUS_REVERSE:
            case self::STATUS_REFUND:
                $payment->setBaseAmountRefunded($payment->getBaseAmountOrdered());
                $payment->setAmountRefunded($payment->getAmountOrdered());
                $order->setTotalRefunded($payment->getAmountOrdered());
                $order->setBaseTotalRefunded($payment->getBaseAmountOrdered());
                $transaction = self::create_transaction($order, $transaction_builder, $_forteecom_order_id, Transaction::TYPE_REFUND);
                $new_state = Order::STATE_CLOSED;
                break;
            case self::STATUS_CANCEL:
                $payment->setBaseAmountCanceled($payment->getBaseAmountOrdered());
                $payment->setAmountCanceled($payment->getAmountOrdered());
                $order->setTotalCanceled($payment->getAmountOrdered());
                $order->setBaseTotalCanceled($payment->getBaseAmountOrdered());
                $new_state = Order::STATE_CANCELED;
                break;
            case self::STATUS_ON_HOLD:
                $payment->setBaseAmountAuthorized($payment->getBaseAmountOrdered());
                $payment->setAmountAuthorized($payment->getAmountOrdered());
                $order->setPaymentAuthorizationAmount($payment->getAmountOrdered());
                self::create_transaction($order, $transaction_builder, $_forteecom_order_id, Transaction::TYPE_AUTH);
                $new_state = Order::STATE_PROCESSING;
                break;
        }

        if (isset($new_state)) {
            $order->setState($new_state);
            $order->addCommentToStatusHistory('ForteEcom: State changed to ' . $new_state, true);
            $payment->save();
            $order->save();
        }
    }

    /**
     * @param Order            $order
     * @param BuilderInterface $transaction_builder
     * @param                  $_forteecom_order_id
     * @param                  $transaction_type
     * @param int              $closed
     * @param string|null      $parent_transaction_id
     * @param int|null         $parent_id
     */
    public static function create_transaction(&$order, $transaction_builder, $_forteecom_order_id, $transaction_type, $closed = 0, $parent_transaction_id = null, $parent_id = null)
    {
        $payment = $order->getPayment();
        $payment->setLastTransId($_forteecom_order_id);
        $payment->setTransactionId($_forteecom_order_id);
        $formatedPrice = $order->getBaseCurrency()->formatTxt(
            $order->getGrandTotal()
        );

        $message = __('Transaction %1 created for amount %2.', $transaction_type, $formatedPrice);

        $transaction = $transaction_builder->setPayment($payment)
            ->setOrder($order)
            ->setTransactionId($_forteecom_order_id)
            ->setFailSafe(true)
            ->build($transaction_type)
            ->setIsClosed($closed);

        if (isset($parent_transaction_id)) {
            $transaction->setParentTxnId($parent_transaction_id);
        }
        if (isset($parent_id)) {
            $transaction->setParentId($parent_id);
        }

        $payment->addTransactionCommentsToOrder(
            $transaction,
            $message
        );
        $payment->setParentTransactionId(null);
        $transaction->save();

        return $transaction;
    }

    /**
     * @param string                $new_status
     * @param ManagerInterface|null $message_manager
     */
    public static function process_payment_status($status, $message_manager)
    {
        switch ($status) {
            case self::STATUS_SUCCESS:
                $message_success = _('Payment complete');
                break;
            case self::STATUS_DECLINE:
                $message_error = __('Payment declined by Gateway');
                break;
            case self::STATUS_EXPIRE:
                $message_error = __('Payment session is Expired');
                break;
            case self::STATUS_REVERSE:
                $message_error = __('Payment reversed');
                break;
            case self::STATUS_REFUND:
                $message_error = __('Payment refunded');
                break;
            case self::STATUS_CANCEL:
                $message_error = __('Payment canceled');
                break;
            case self::STATUS_ON_HOLD:
                $message_success = _('Payment complete and on-hold');
                break;
        }

        if (isset($message_success)) {
            $message_manager->addSuccessMessage($message_success);
        }
        if (isset($message_error)) {
            $message_manager->addErrorMessage($message_error);
        }

    }

    public static function get_available_currency_names()
    {
        $currencies = [];
        foreach (self::AVAILABLE_CURRENCIES as $currency) {
            $currencies[] = $currency['name'];
        }
        return implode(', ', $currencies);
    }

    public static function get_url($endpoint)
    {
        $host = (FORTEECOM_DEMO_ENABLED) ? self::HOST_DEMO : self::HOST;
        return $host . '/ecom' . $endpoint;
    }

    public static function get_script() {
        if (FORTEECOM_DEMO_ENABLED) {
            //$cdn_url = 'https://cdn-1.forte.kz/assets/plugins/forte-ecom-qa/v1/ecom-widget-feature-plugin/';
            $cdn_url = 'https://cdn-1.forte.kz/assets/plugins/forte-ecom-qago/v1/ecom-widget-feature-plugin/';
        } else {
            $cdn_url = 'https://cdn-1.forte.kz/assets/plugins/forte-ecom/v1/ecom-widget-feature-plugin/';
        }

        return '<script type="module" src="' . $cdn_url . 'ecom-widget-feature-plugin.esm.js?ver=' . time() . '"></script>
        <script nomodule="" src="' . $cdn_url . 'ecom-widget-feature-plugin.js?ver=' . time() . '"></script>';
    }

}
